@@ -177,7 +177,7 @@ fromDataFrame <- function(obj, uri, col_index=NULL, sparse=FALSE, allows_dups=sp
         else if (cl == "numeric")
             tp <- "FLOAT64"
         else if (cl == "character")
-            tp <- "CHAR"
+            tp <- "ASCII"
         else if (cl == "Date")
             tp <- "DATETIME_DAY"
         else if (cl == "POSIXct" || cl == "POSIXlt")
@@ -193,21 +193,20 @@ fromDataFrame <- function(obj, uri, col_index=NULL, sparse=FALSE, allows_dups=sp
         }
         tiledb_attr(colnames(obj)[ind],
                     type = tp,
-                    ncells = ifelse(tp=="CHAR",NA_integer_,1),
+                    ncells = ifelse(tp %in% c("CHAR","ASCII"), NA_integer_, 1),
                     filter_list = filterlist,
                     nullable = any(is.na(col)))
     }
     cols <- seq_len(dims[2])
     if (!is.null(col_index)) cols <- cols[-col_index]
     attributes <- sapply(cols, makeAttr)
-
     schema <- tiledb_array_schema(dom, attrs = attributes,
                                   cell_order = cell_order, tile_order = tile_order,
                                   sparse=sparse, capacity=capacity)
     allows_dups(schema) <- allows_dups
     tiledb_array_create(uri, schema)
 
-    df <- tiledb_array(uri)
+    df <- tiledb_array(uri, query_type = "WRITE")
     ## when setting an index when likely want 'sparse write to dense array
     if (!is.null(col_index) && !sparse) query_layout(df) <- "UNORDERED"
     if (is.null(col_index) && sparse)
