@@ -28,6 +28,10 @@
 #' @slot attrs A character vector
 #' @slot extended A logical value
 #' @slot ptr External pointer to the underlying implementation
+#' @section Planned Deprecation:
+#' We plan to deprecate the \code{tiledb_sparse} array type in a future release. While exact
+#' timelines have not been finalised, it is advised to the \code{tiledb_array} for both
+#' \emph{dense} and \emph{sparse} arrays going forward.
 #' @exportClass tiledb_sparse
 setClass("tiledb_sparse",
          slots = list(ctx = "tiledb_ctx",
@@ -41,6 +45,10 @@ setClass("tiledb_sparse",
 #'
 #' tiledb_sparse returns a list of coordinates and attributes vectors for reads
 #'
+#' @section Planned Deprecation:
+#' We plan to deprecate the \code{tiledb_sparse} array type in a future release. While exact
+#' timelines have not been finalised, it is advised to the \code{tiledb_array} for both
+#' \emph{dense} and \emph{sparse} arrays going forward.
 #' @param uri uri path to the tiledb dense array
 #' @param query_type optionally loads the array in "READ" or "WRITE" only modes.
 #' @param as.data.frame optional logical switch, defaults to "FALSE"
@@ -63,7 +71,7 @@ tiledb_sparse <- function(uri,
   } else if (missing(uri) || !is.scalar(uri, "character")) {
     stop("argument uri must be a string scalar")
   }
-
+  .Deprecated("tiledb_array")
   array_xptr <- libtiledb_array_open(ctx@ptr, uri, query_type)
   schema_xptr <- libtiledb_array_get_schema(array_xptr)
   if (!libtiledb_array_schema_sparse(schema_xptr)) {
